=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2011-05-03 22:53:39 +0000
+++ bzrlib/bzrdir.py	2011-05-03 23:16:56 +0000
@@ -1252,15 +1252,12 @@
     def has_workingtree(self):
         """Tell if this bzrdir contains a working tree.
 
-        This will still raise an exception if the bzrdir has a workingtree that
-        is remote & inaccessible.
-
         Note: if you're going to open the working tree, you should just go
         ahead and try, and not ask permission first.
         """
         from bzrlib.workingtree import WorkingTreeFormat
         try:
-            WorkingTreeFormat.find_format(self)
+            WorkingTreeFormat.find_format_string(self)
         except errors.NoWorkingTree:
             return False
         return True

=== modified file 'bzrlib/tests/blackbox/test_exceptions.py'
--- bzrlib/tests/blackbox/test_exceptions.py	2011-04-15 18:11:36 +0000
+++ bzrlib/tests/blackbox/test_exceptions.py	2011-05-03 22:59:49 +0000
@@ -102,7 +102,7 @@
         bzrdir.register_metadir(controldir.format_registry, "testobsolete",
             "bzrlib.tests.blackbox.test_exceptions.TestObsoleteRepoFormat",
             branch_format='bzrlib.branch.BzrBranchFormat7',
-            tree_format='bzrlib.workingtree.WorkingTreeFormat6',
+            tree_format='bzrlib.workingtree_4.WorkingTreeFormat6',
             deprecated=True,
             help='Same as 2a, but with an obsolete repo format.')
         self.disable_deprecation_warning()

=== modified file 'bzrlib/tests/test_import_tariff.py'
--- bzrlib/tests/test_import_tariff.py	2011-05-03 22:22:59 +0000
+++ bzrlib/tests/test_import_tariff.py	2011-05-03 23:16:56 +0000
@@ -206,6 +206,9 @@
             'bzrlib.bugtracker',
             'bzrlib.bundle.commands',
             'bzrlib.cmd_version_info',
+            'bzrlib.dirstate',
+            'bzrlib._dirstate_helpers_py',
+            'bzrlib._dirstate_helpers_pyx',
             'bzrlib.externalcommand',
             'bzrlib.filters',
             # foreign branch plugins import the foreign_vcs_registry from 
@@ -223,6 +226,7 @@
             'bzrlib.smart.client',
             'bzrlib.transform',
             'bzrlib.version_info_formats.format_rio',
+            'bzrlib.workingtree_4',
             'getpass',
             'kerberos',
             'smtplib',

=== modified file 'bzrlib/tests/test_selftest.py'
--- bzrlib/tests/test_selftest.py	2011-04-15 07:01:22 +0000
+++ bzrlib/tests/test_selftest.py	2011-05-04 01:42:10 +0000
@@ -52,6 +52,7 @@
     tests,
     transport,
     workingtree,
+    workingtree_4,
     )
 from bzrlib.repofmt import (
     groupcompress_repo,
@@ -338,7 +339,7 @@
         from bzrlib.tests.per_workingtree import make_scenarios
         server1 = "a"
         server2 = "b"
-        formats = [workingtree.WorkingTreeFormat4(),
+        formats = [workingtree_4.WorkingTreeFormat4(),
                    workingtree.WorkingTreeFormat3(),]
         scenarios = make_scenarios(server1, server2, formats)
         self.assertEqual([
@@ -375,13 +376,13 @@
             )
         server1 = "a"
         server2 = "b"
-        formats = [workingtree.WorkingTreeFormat4(),
+        formats = [workingtree_4.WorkingTreeFormat4(),
                    workingtree.WorkingTreeFormat3(),]
         scenarios = make_scenarios(server1, server2, formats)
         self.assertEqual(7, len(scenarios))
         default_wt_format = workingtree.format_registry.get_default()
-        wt4_format = workingtree.WorkingTreeFormat4()
-        wt5_format = workingtree.WorkingTreeFormat5()
+        wt4_format = workingtree_4.WorkingTreeFormat4()
+        wt5_format = workingtree_4.WorkingTreeFormat5()
         expected_scenarios = [
             ('WorkingTreeFormat4',
              {'bzrdir_format': formats[0]._matchingbzrdir,
@@ -453,7 +454,8 @@
         from bzrlib.tests.per_intertree import (
             make_scenarios,
             )
-        from bzrlib.workingtree import WorkingTreeFormat3, WorkingTreeFormat4
+        from bzrlib.workingtree import WorkingTreeFormat3
+        from bzrlib.workingtree_4 import WorkingTreeFormat4
         input_test = TestInterTreeScenarios(
             "test_scenarios")
         server1 = "a"

=== modified file 'bzrlib/tests/test_workingtree.py'
--- bzrlib/tests/test_workingtree.py	2011-05-03 22:53:39 +0000
+++ bzrlib/tests/test_workingtree.py	2011-05-03 23:16:56 +0000
@@ -166,6 +166,19 @@
 class TestWorkingTreeFormat(TestCaseWithTransport):
     """Tests for the WorkingTreeFormat facility."""
 
+    def test_find_format_string(self):
+        # is the right format object found for a working tree?
+        branch = self.make_branch('branch')
+        self.assertRaises(errors.NoWorkingTree,
+            workingtree.WorkingTreeFormat.find_format_string, branch.bzrdir)
+        transport = branch.bzrdir.get_workingtree_transport(None)
+        transport.mkdir('.')
+        transport.put_bytes("format", "some format name")
+        # The format does not have to be known by Bazaar,
+        # find_format_string just retrieves the name
+        self.assertEquals("some format name",
+            workingtree.WorkingTreeFormat.find_format_string(branch.bzrdir))
+
     def test_find_format(self):
         # is the right format object found for a working tree?
         # create a branch with a few known format objects.

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2011-05-03 22:53:39 +0000
+++ bzrlib/workingtree.py	2011-05-04 08:49:15 +0000
@@ -2978,10 +2978,12 @@
         return self._default_format
 
     def set_default(self, format):
+        """Set the default format."""
         self._default_format = format
         self._default_format_key = None
 
     def set_default_key(self, format_string):
+        """Set the default format by its format string."""
         self._default_format_key = format_string
         self._default_format = None
 
@@ -3019,14 +3021,20 @@
     """If this format supports missing parent conflicts."""
 
     @classmethod
+    def find_format_string(klass, a_bzrdir):
+        """Return format name for the working tree object in a_bzrdir."""
+        try:
+            transport = a_bzrdir.get_workingtree_transport(None)
+            return transport.get_bytes("format")
+        except errors.NoSuchFile:
+            raise errors.NoWorkingTree(base=transport.base)
+
+    @classmethod
     def find_format(klass, a_bzrdir):
         """Return the format for the working tree object in a_bzrdir."""
         try:
-            transport = a_bzrdir.get_workingtree_transport(None)
-            format_string = transport.get_bytes("format")
+            format_string = klass.find_format_string(a_bzrdir)
             return format_registry.get(format_string)
-        except errors.NoSuchFile:
-            raise errors.NoWorkingTree(base=transport.base)
         except KeyError:
             raise errors.UnknownFormatError(format=format_string,
                                             kind="working tree")

